class RunsController < ApplicationController

  before_filter :authorize

  def index
    list
    render :action => 'list'
  end

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def list
    @runs = Run.find_all_by_user_id(session[:user_id]).sort
  end

  def show
    begin
      run = Run.find(params[:id])
    rescue Activerecord::RecordNotFound
      redirect_to_index('Invalid run')
    else
      @run = run
    end
  end

  def new
    @run = Run.new
  end

  def create
    @run = Run.new(params[:run])
    if @run.save
      flash[:notice] = 'Run was successfully created.'
      redirect_to :action => 'list'
    else
      render :action => 'new'
    end
  end

  def edit
    @run = Run.find(params[:id])
  end

  def update
    @run = Run.find(params[:id])
    if @run.update_attributes(params[:run])
      flash[:notice] = 'Run was successfully updated.'
      redirect_to :action => 'show', :id => @run
    else
      render :action => 'edit'
    end
  end

  def destroy
    Run.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
  
  def show_upload
  end

  def upload
    require 'rexml/document'
    require 'time'
        
    xml_file = params[:xml_doc][:xml_file]
    xml_doc = REXML::Document.new(xml_file.read)
    root = xml_doc.root

    if root.elements["vers"][0] != 2
      flash.now[:notice] = "Wrong version of Nike XML data"
      #redirect_to(:action => 'index')
      return
    end

    #OK, time to make a run object that contains all the crazy info in this XML doc
    @run = Run.new
    @run.user_id = session[:user_id]
    @run.start_time = Time.xmlschema(root.elements["runSummary/time"][0].value) #Start time of run
    @run.duration = root.elements["runSummary/duration"][0].value #Duration in ms
    @run.playlist = root.elements["runSummary/playlistList/playlist/playlistName"][0].value unless root.elements["runSummary/playlistList/playlist/playlistName"].nil? # Playlist name
    @run.weight = root.elements["userInfo/weight"][0].value
    @run.device = root.elements["userInfo/device"][0].value
    @run.calibration = root.elements["userInfo/calibration"][0].value

    #run_distance = root.elements["runSummary/distance"][0].value
    #run_distance_units = root.elements["runSummary/distance"].attributes["unit"]
    @run.save

    #Make a snapshot object for each snapshot in the run (mile/km splits, pause, resume, etc)
    root.elements.each("snapShotList[@snapShotType='kmSplit']/snapShot") do |element| 
      @snapshot = SnapShot.new
      @snapshot.run_id = @run.id
      @snapshot.snapshot_type = 'kmSplit'
      @snapshot.duration = element.elements["duration"][0].value
      @snapshot.distance = element.elements["distance"][0].value
      @snapshot.pace = element.elements["pace"][0].value
      @snapshot.save
    end

    root.elements.each("snapShotList[@snapShotType='mileSplit']/snapShot") do |element| 
      @snapshot = SnapShot.new
      @snapshot.run_id = @run.id
      @snapshot.snapshot_type = 'mileSplit'
      @snapshot.duration = element.elements["duration"][0].value
      @snapshot.distance = element.elements["distance"][0].value
      @snapshot.pace = element.elements["pace"][0].value
      @snapshot.save
    end

    root.elements.each("snapShotList[@snapShotType='userClick']/snapShot") do |element| 
      @snapshot = SnapShot.new
      @snapshot.run_id = @run.id
      @snapshot.snapshot_type = 'userClick'
      @snapshot.event_type = element.attributes["event"]
      @snapshot.duration = element.elements["duration"][0].value
      @snapshot.distance = element.elements["distance"][0].value
      @snapshot.pace = element.elements["pace"][0].value
      @snapshot.save
    end

    #Save extended data (splits at 10 second intervals)
    @extendeddata = ExtendedData.new
    @extendeddata.run_id = @run.id
    @extendeddata.data_type = root.elements["extendedDataList/extendedData"].attributes["dataType"]
    @extendeddata.interval_type = root.elements["extendedDataList/extendedData"].attributes["intervalType"]
    @extendeddata.interval_unit = root.elements["extendedDataList/extendedData"].attributes["intervalUnit"]
    @extendeddata.interval_value = root.elements["extendedDataList/extendedData"].attributes["intervalValue"]
    @extendeddata.extended_data = root.elements["extendedDataList/extendedData"][0].value
    @extendeddata.save

    #render :action => index
    #redirect_to :action => index
  end
end
